package eu.jonahbauer.raytracing.scene;

import eu.jonahbauer.raytracing.math.Range;
import eu.jonahbauer.raytracing.math.Ray;
import eu.jonahbauer.raytracing.math.Vec3;
import eu.jonahbauer.raytracing.render.material.Material;
import eu.jonahbauer.raytracing.render.texture.Texture;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

/**
 * The result of a {@linkplain Hittable#hit(Ray, Range) hit}.
 * @param t the {@code t} value at which the hit occurs
 * @param position the position of the hit
 * @param normal the surface normal at the hit position
 * @param target the hit target (for debug purposes only)
 * @param material the material of the surface
 * @param u the texture u coordinate (or {@code Double.NaN} if the {@linkplain Material#texture() material's texture} does {@linkplain Texture#isUVRequired() not depend} on the uv-coordinates)
 * @param v the texture v coordinate (or {@code Double.NaN} if the {@linkplain Material#texture() material's texture} does {@linkplain Texture#isUVRequired() not depend} on the uv-coordinates)
 * @param isFrontFace whether the front or the back of the surface was it
 */
public record HitResult(
        double t, @NotNull Vec3 position, @NotNull Vec3 normal, @NotNull Hittable target,
        @NotNull Material material, double u, double v, boolean isFrontFace
) implements Comparable<HitResult> {
    public HitResult {
        Objects.requireNonNull(position, "position");
        normal = normal.unit();
    }

    public @NotNull HitResult withPositionAndNormal(@NotNull Vec3 position, @NotNull Vec3 normal) {
        return new HitResult(t, position, normal, target, material, u, v, isFrontFace);
    }

    @Override
    public int compareTo(@NotNull HitResult o) {
        return Double.compare(t, o.t);
    }
}
